@@ -3,6 +3,7 @@
 
 #pragma pack(1)
 
+#define ERAR_SUCCESS             0
 #define ERAR_END_ARCHIVE        10
 #define ERAR_NO_MEMORY          11
 #define ERAR_BAD_DATA           12
@@ -28,7 +29,12 @@
 #define RAR_VOL_ASK           0
 #define RAR_VOL_NOTIFY        1
 
-#define RAR_DLL_VERSION       5
+#define RAR_DLL_VERSION       6
+
+#define RAR_HASH_NONE         0
+#define RAR_HASH_CRC32        1
+#define RAR_HASH_BLAKE2       2
+
 
 #ifdef _UNIX
 #define CALLBACK
@@ -39,6 +45,13 @@
 #define UINT unsigned int
 #endif
 
+#define RHDF_SPLITBEFORE 0x01
+#define RHDF_SPLITAFTER  0x02
+#define RHDF_ENCRYPTED   0x04
+#define RHDF_SOLID       0x10
+#define RHDF_DIRECTORY   0x20
+
+
 struct RARHeaderData
 {
   char         ArcName[260];
@@ -80,7 +93,10 @@ struct RARHeaderDataEx
   unsigned int CmtBufSize;
   unsigned int CmtSize;
   unsigned int CmtState;
-  unsigned int Reserved[1024];
+  unsigned int DictSize;
+  unsigned int HashType;
+  char         Hash[32];
+  unsigned int Reserved[1014];
 };
 
 
@@ -114,7 +130,8 @@ struct RAROpenArchiveDataEx
 };
 
 enum UNRARCALLBACK_MESSAGES {
-  UCM_CHANGEVOLUME,UCM_PROCESSDATA,UCM_NEEDPASSWORD
+  UCM_CHANGEVOLUME,UCM_PROCESSDATA,UCM_NEEDPASSWORD,UCM_CHANGEVOLUMEW,
+  UCM_NEEDPASSWORDW
 };
 
 typedef int (PASCAL *CHANGEVOLPROC)(char *ArcName,int Mode);
