@@ -6,7 +6,7 @@
 #define   MRetryAbort        "_Retry_Abort"
 #define   MCopyright         "\nRAR %s   Copyright (c) 1993-%d Alexander Roshal   %d %s %d"
 #define   MRegTo             "\nRegistered to %s\n"
-#define   MShare             "\nShareware version         Type RAR -? for help\n"
+#define   MShare             "\nTrial version             Type RAR -? for help\n"
 #define   MUCopyright        "\nUNRAR %s freeware      Copyright (c) 1993-%d Alexander Roshal\n"
 #define   MBeta              "beta"
 #define   MMonthJan          "Jan"
@@ -27,15 +27,14 @@
 #define   MCHelpCmd          "\n\n<Commands>"
 #define   MCHelpCmdA         "\n  a             Add files to archive"
 #define   MCHelpCmdC         "\n  c             Add archive comment"
-#define   MCHelpCmdCF        "\n  cf            Add files comment"
 #define   MCHelpCmdCH        "\n  ch            Change archive parameters"
 #define   MCHelpCmdCW        "\n  cw            Write archive comment to file"
 #define   MCHelpCmdD         "\n  d             Delete files from archive"
-#define   MCHelpCmdE         "\n  e             Extract files to current directory"
+#define   MCHelpCmdE         "\n  e             Extract files without archived paths"
 #define   MCHelpCmdF         "\n  f             Freshen files in archive"
 #define   MCHelpCmdI         "\n  i[par]=<str>  Find string in archives"
 #define   MCHelpCmdK         "\n  k             Lock archive"
-#define   MCHelpCmdL         "\n  l[t,b]        List archive [technical, bare]"
+#define   MCHelpCmdL         "\n  l[t[a],b]     List archive contents [technical[all], bare]"
 #define   MCHelpCmdM         "\n  m[f]          Move to archive [files only]"
 #define   MCHelpCmdP         "\n  p             Print file to stdout"
 #define   MCHelpCmdR         "\n  r             Repair archive"
@@ -46,10 +45,11 @@
 #define   MCHelpCmdS         "\n  s[name|-]     Convert archive to or from SFX"
 #define   MCHelpCmdT         "\n  t             Test archive files"
 #define   MCHelpCmdU         "\n  u             Update files in archive"
-#define   MCHelpCmdV         "\n  v[t,b]        Verbosely list archive [technical,bare]"
+#define   MCHelpCmdV         "\n  v[t[a],b]     Verbosely list archive contents [technical[all],bare]"
 #define   MCHelpCmdX         "\n  x             Extract files with full path"
 #define   MCHelpSw           "\n\n<Switches>"
 #define   MCHelpSwm          "\n  -             Stop switches scanning"
+#define   MCHelpSwAT         "\n  @[+]          Disable [enable] file lists"
 #define   MCHelpSwAC         "\n  ac            Clear Archive attribute after compression or extraction"
 #define   MCHelpSwAD         "\n  ad            Append archive name to destination path"
 #define   MCHelpSwAG         "\n  ag[format]    Generate archive name using the current date"
@@ -57,8 +57,6 @@
 #define   MCHelpSwAO         "\n  ao            Add files with Archive attribute set"
 #define   MCHelpSwAP         "\n  ap<path>      Set path inside archive"
 #define   MCHelpSwAS         "\n  as            Synchronize archive contents"
-#define   MCHelpSwAV         "\n  av            Put authenticity verification (registered versions only)"
-#define   MCHelpSwAVm        "\n  av-           Disable authenticity verification check"
 #define   MCHelpSwCm         "\n  c-            Disable comments show"
 #define   MCHelpSwCFGm       "\n  cfg-          Disable read configuration"
 #define   MCHelpSwCL         "\n  cl            Convert names to lower case"
@@ -78,6 +76,7 @@
 #define   MCHelpSwEP3        "\n  ep3           Expand paths to full including the drive letter"
 #define   MCHelpSwF          "\n  f             Freshen files"
 #define   MCHelpSwHP         "\n  hp[password]  Encrypt both file data and headers"
+#define   MCHelpSwHT         "\n  ht[b|c]       Select hash type [BLAKE2,CRC32] for file checksum"
 #define   MCHelpSwIDP        "\n  id[c,d,p,q]   Disable messages"
 #define   MCHelpSwIEML       "\n  ieml[addr]    Send archive by email"
 #define   MCHelpSwIERR       "\n  ierr          Send all messages to stderr"
@@ -87,22 +86,27 @@
 #define   MCHelpSwISND       "\n  isnd          Enable sound"
 #define   MCHelpSwK          "\n  k             Lock archive"
 #define   MCHelpSwKB         "\n  kb            Keep broken extracted files"
+#define   MCHelpSwLog        "\n  log[f][=name] Write names to log file"
 #define   MCHelpSwMn         "\n  m<0..5>       Set compression level (0-store...3-default...5-maximal)"
+#define   MCHelpSwMA         "\n  ma[4|5]       Specify a version of archiving format"
 #define   MCHelpSwMC         "\n  mc<par>       Set advanced compression parameters"
-#define   MCHelpSwMD         "\n  md<size>      Dictionary size in KB (64,128,256,512,1024,2048,4096 or A-G)"
+#define   MCHelpSwMD         "\n  md<n>[k,m,g]  Dictionary size in KB, MB or GB"
 #define   MCHelpSwMS         "\n  ms[ext;ext]   Specify file types to store"
 #define   MCHelpSwMT         "\n  mt<threads>   Set the number of threads"
-#define   MCHelpSwN          "\n  n<file>       Include only specified file"
-#define   MCHelpSwNa         "\n  n@            Read file names to include from stdin"
-#define   MCHelpSwNal        "\n  n@<list>      Include files listed in specified list file"
+#define   MCHelpSwN          "\n  n<file>       Additionally filter included files"
+#define   MCHelpSwNa         "\n  n@            Read additional filter masks from stdin"
+#define   MCHelpSwNal        "\n  n@<list>      Read additional filter masks from list file"
 #define   MCHelpSwO          "\n  o[+|-]        Set the overwrite mode"
 #define   MCHelpSwOC         "\n  oc            Set NTFS Compressed attribute"
+#define   MCHelpSwOH         "\n  oh            Save hard links as the link instead of the file"
+#define   MCHelpSwOI         "\n  oi[0-4][:min] Save identical files as references"
 #define   MCHelpSwOL         "\n  ol            Save symbolic links as the link instead of the file"
 #define   MCHelpSwOR         "\n  or            Rename files automatically"
 #define   MCHelpSwOS         "\n  os            Save NTFS streams"
 #define   MCHelpSwOW         "\n  ow            Save or restore file owner and group"
 #define   MCHelpSwP          "\n  p[password]   Set password"
 #define   MCHelpSwPm         "\n  p-            Do not query password"
+#define   MCHelpSwQO         "\n  qo[-|+]       Add quick open information [none|force]"
 #define   MCHelpSwR          "\n  r             Recurse subdirectories"
 #define   MCHelpSwRm         "\n  r-            Disable recursion"
 #define   MCHelpSwR0         "\n  r0            Recurse subdirectories for wildcard names only"
@@ -140,9 +144,8 @@
 #define   MCHelpSwZ          "\n  z[file]       Read archive comment from file"
 #define   MBadArc            "\nERROR: Bad archive %s\n"
 #define   MAskPsw            "Enter password (will not be echoed)"
-#define   MAskPswEcho        "Enter password"
+#define   MAskPswFor         "\nEnter password (will not be echoed) for %s: "
 #define   MReAskPsw          "\nReenter password: "
-#define   MFor               " for "
 #define   MNotMatchPsw       "\nERROR: Passwords do not match\n"
 #define   MErrWrite          "Write error in the file %s"
 #define   MErrRead           "Read error in the file %s"
@@ -171,20 +174,13 @@
 #define   MCannotOpen        "\nCannot open %s"
 #define   MUnknownMeth       "\nUnknown method in %s"
 #define   MVerRequired       "\nYou need RAR %d.%d to unpack it"
+#define   MNewRarFormat      "\nUnsupported archive format. Please update RAR to a newer version."
 #define   MOk                " OK"
 #define   MDone              "\nDone"
 #define   MLockingArc        "\nLocking archive"
 #define   MNotMdfOld         "\n\nERROR: Cannot modify old format archive"
 #define   MNotMdfLock        "\n\nERROR: Locked archive"
 #define   MNotMdfVol         "\n\nERROR: Cannot modify volume"
-#define   MVerifyAV          "\nVerifying authenticity information ... "
-#define   MFailedAV          " Failed\n"
-#define   MStrAV1            "\n\nArchive %s"
-#define   MStrAV2            "\ncreated at %s"
-#define   MStrAV3            "\nby %s\n"
-#define   MLogFailedAV       "Invalid authenticity information"
-#define   MAddingAV          "\nAdding authenticity verification "
-#define   MAVOldStyle        "\n\nOld style authenticity information"
 #define   MPackAskReg        "\nEvaluation copy. Please register.\n"
 #define   MCreateArchive     "\nCreating %sarchive %s\n"
 #define   MUpdateArchive     "\nUpdating %sarchive %s\n"
@@ -208,7 +204,7 @@
 #define   MCannotMdfEncrSol  "\nCannot modify solid archive containing encrypted files"
 #define   MAddAnalyze        "\nAnalyzing archived files: "
 #define   MRepacking         "\nRepacking archived files: "
-#define   MCRCFailed         "\n%-20s - CRC failed"
+#define   MCRCFailed         "\n%-20s - checksum error"
 #define   MExtrTest          "\n\nTesting archive %s\n"
 #define   MExtracting        "\n\nExtracting from %s\n"
 #define   MUseCurPsw         "\n%s - use current password ?"
@@ -219,14 +215,15 @@
 #define   MExtrPoints        "\n...         %-56s"
 #define   MExtrErrMkDir      "\nCannot create directory %s"
 #define   MExtrPrinting      "\n------ Printing %s\n\n"
-#define   MEncrBadCRC        "\nCRC failed in the encrypted file %s. Corrupt file or wrong password."
+#define   MEncrBadCRC        "\nChecksum error in the encrypted file %s. Corrupt file or wrong password."
 #define   MExtrNoFiles       "\nNo files to extract"
 #define   MExtrAllOk         "\nAll OK"
 #define   MExtrTotalErr      "\nTotal errors: %ld"
 #define   MFileExists        "\n\n%s already exists. Overwrite it ?"
 #define   MAskOverwrite      "\nOverwrite %s ?"
 #define   MAskNewName        "\nEnter new name: "
-#define   MLogMainHead       "\nThe archive header is corrupt"
+#define   MHeaderBroken      "\nCorrupt header is found"
+#define   MMainHeaderBroken  "\nMain archive header is corrupt"
 #define   MLogFileHead       "\n%s - the file header is corrupt"
 #define   MLogCommHead       "\nThe comment header is corrupt\n"
 #define   MLogProtectHead    "The data recovery header is corrupt"
@@ -238,7 +235,6 @@
 #define   MAskFComm          "\n\nReading comment for %s : %s from stdin\n"
 #define   MLogCommBrk        "\nThe archive comment is corrupt"
 #define   MCommAskCont       "\nPress 'Enter' to continue or 'Q' to quit:"
-#define   MLogBrokFCmt       "\nThe file comment is corrupt"
 #define   MWriteCommTo       "\nWrite comment to %s"
 #define   MCommNotPres       "\nComment is not present"
 #define   MDelFrom           "\nDeleting from %s"
@@ -247,18 +243,40 @@
 #define   MNoDelFiles        "\nNo files to delete"
 #define   MLogTitle          "\n\n--------  %2d %s %d, archive %s\n"
 #define   MPathTooLong       "\nERROR: Path too long\n"
-#define   MListSolid         "Solid "
-#define   MListSFX           "SFX "
-#define   MListVol1          "volume"
-#define   MListVol2          "Volume"
-#define   MListArc1          "archive"
-#define   MListArc2          "Archive"
-#define   MListRecRec        "\nRecovery record is present\n"
-#define   MListLock          "\nLock is present\n"
-#define   MListPathComm      "\nPathname/Comment\n            "
-#define   MListName          "\n Name       "
-#define   MListTitle         "      Size   Packed Ratio  Date   Time     Attr      CRC   Meth Ver\n"
-#define   MListTechTitle     "               Host OS    Solid   Old\n"
+#define   MListArchive       "Archive"
+#define   MListDetails       "Details"
+#define   MListSolid         "solid"
+#define   MListSFX           "SFX"
+#define   MListVolume        "volume"
+#define   MListRR            "recovery record"
+#define   MListLock          "lock"
+#define   MListEnc           "encrypted"
+#define   MListEncHead       "encrypted headers"
+#define   MListTitleL        " Attributes      Size    Date   Time   Name"
+#define   MListTitleV        " Attributes      Size    Packed Ratio   Date   Time   Checksum  Name"
+#define   MListName          "Name"
+#define   MListType          "Type"
+#define   MListFile          "File"
+#define   MListDir           "Directory"
+#define   MListUSymlink      "Unix symbolic link"
+#define   MListWSymlink      "Windows symbolic link"
+#define   MListJunction      "NTFS junction point"
+#define   MListHardlink      "Hard link"
+#define   MListCopy          "File copy"
+#define   MListStream        "NTFS alternate data stream"
+#define   MListTarget        "Target"
+#define   MListSize          "Size"
+#define   MListPacked        "Packed size"
+#define   MListRatio         "Ratio"
+#define   MListMtime         "mtime"
+#define   MListCtime         "ctime"
+#define   MListAtime         "atime"
+#define   MListAttr          "Attributes"
+#define   MListFlags         "Flags"
+#define   MListCompInfo      "Compression"
+#define   MListHostOS        "Host OS"
+#define   MListFileVer       "File version"
+#define   MListService       "Service"
 #define   MListEAHead        "\n   OS/2 extended attributes"
 #define   MListUOHead        "\n   Unix Owner/Group data:    %-14s %-14s"
 #define   MListBeEAHead      "\n   BeOS extended attributes"
@@ -275,10 +293,6 @@
 #define   MRprFind           "\nFound  %s"
 #define   MRprAskIsSol       "\nThe archive header is corrupt. Mark archive as solid ?"
 #define   MRprNoFiles        "\nNo files found"
-#define   MRprSuspEntry      "\n\nSuspicious entry %s"
-#define   MRprDir            "\nDirectory"
-#define   MRprSuspSize       "\nSize %ld  Packed %ld"
-#define   MRprSuspAdd        "\nAdd it to archive ?"
 #define   MLogUnexpEOF       "\nUnexpected end of archive"
 #define   MRepAskReconst     "\nReconstruct archive structure ?"
 #define   MRecScanning       "\nScanning..."
@@ -295,12 +309,11 @@
 #define   MOwnersBroken      "\nERROR: %s group and owner data are corrupt\n"
 #define   MSetOwnersError    "\nWARNING: Cannot set %s owner and group\n"
 #define   MErrLnkRead        "\nWARNING: Cannot read symbolic link %s"
-#define   MErrCreateLnk      "\nWARNING: Cannot create link %s"
 #define   MSymLinkExists     "\nWARNING: Symbolic link %s already exists"
 #define   MAskRetryCreate    "\nCannot create %s. Retry ?"
 #define   MListMACHead1      "\n Mac OS file type:  %c%c%c%c  ; "
 #define   MListMACHead2      "file creator:  %c%c%c%c\n"
-#define   MDataBadCRC        "\n%-20s : packed data CRC failed in volume %s"
+#define   MDataBadCRC        "\n%-20s : packed data checksum error in volume %s"
 #define   MFileRO            "\n%s is read-only"
 #define   MACLGetError       "\nWARNING: Cannot get %s security data\n"
 #define   MACLSetError       "\nWARNING: Cannot set %s security data\n"
@@ -345,10 +358,31 @@
 #define   MSHelpCmdT         "\n  -t      Test archive files" 
 #define   MSHelpCmdV         "\n  -v      Verbosely list contents of archive" 
 #define   MMaxPathLimit      "\nTotal path and file name length must not exceed %d characters"
-#define   MRecVolLimit       "\nTotal number of usual and recovery volumes must not exceed 255"
+#define   MRecVolLimit       "\nTotal number of usual and recovery volumes must not exceed %d"
 #define   MVolumeNumber      "volume %d"
 #define   MCannotDelete      "\nCannot delete %s"
-#define   MCalcCRC           "\nCalculating the control sum"
+#define   MCalcCRC           "\nCalculating the checksum"
 #define   MTooLargeSFXArc    "\nWARNING: Too large SFX archive. Windows cannot run the executable file exceeding 4 GB."
-#define   MCalcCRCAllVol     "\nCalculating control sums of all volumes."
+#define   MCalcCRCAllVol     "\nCalculating checksums of all volumes."
 #define   MNotEnoughDisk     "\nERROR: Not enough disk space for %s."
+#define   MNewerRAR          "\nYou may need a newer version of RAR."
+#define   MUnkEncMethod      "\nUnknown encryption method in %s"
+#define   MWrongPassword     "\nThe specified password is incorrect."
+#define   MRepairing         "\nRepairing"
+#define   MAreaDamaged       "\nCorrupt %d bytes at %08x %08x"
+#define   MBlocksRecovered   "\n%d blocks recovered."
+#define   MRRDamaged         "\nRecovery record is corrupt."
+#define   MTestingRR         "\nTesting the recovery record"
+#define   MFailed            "Failed"
+#define   MIncompatSwitch    "\n%s switch is not supported for RAR %d.x archive format."
+#define   MSearchDupFiles    "\nSearching for identical files"
+#define   MNumFound          "%d found."
+#define   MUnknownExtra      "\nUnknown extra field in %s."
+#define   MCopyError         "\nCannot copy %s to %s."
+#define   MCopyErrorHint     "\nYou need to unpack the entire archive to create file copy entries."
+#define   MCopyingData       "\nCopying data"
+#define   MErrCreateLnkS     "\nCannot create symbolic link %s"
+#define   MErrCreateLnkH     "\nCannot create hard link %s"
+#define   MNeedAdmin         "\nYou may need to run RAR as administrator"
+#define   MDictOutMem        "\nNot enough memory for %d MB compression dictionary, changed to %d MB."
+#define   MUseSmalllerDict   "\nPlease use a smaller compression dictionary."
