@@ -17,14 +17,20 @@ RAROptions::~RAROptions()
 void RAROptions::Init()
 {
   memset(this,0,sizeof(RAROptions));
-  WinSize=0x400000;
+  WinSize=0x2000000;
   Overwrite=OVERWRITE_DEFAULT;
   Method=3;
   MsgStream=MSG_STDOUT;
   ConvertNames=NAMES_ORIGINALCASE;
   ProcessEA=true;
   xmtime=EXTTIME_HIGH3;
-  CurVolNum=0;
   FileSizeLess=INT64NDF;
   FileSizeMore=INT64NDF;
+  HashType=HASH_CRC32;
+#ifdef RAR_SMP
+  Threads=GetNumberOfThreads();
+#endif
+#ifdef USE_QOPEN
+  QOpenMode=QOPEN_AUTO;
+#endif
 }
