@@ -3,6 +3,7 @@
 #include "rarvmtbl.cpp"
 
 RarVM::RarVM()
+:BitInput(true)
 {
   Mem=NULL;
 }
@@ -47,12 +48,12 @@ inline uint RarVM::GetValue(bool ByteMode,uint *Addr)
     if (IS_VM_MEM(Addr))
     {
       byte *B=(byte *)Addr;
-      return UINT32((uint)B[0]|((uint)B[1]<<8)|((uint)B[2]<<16)|((uint)B[3]<<24));
+      return GET_UINT32((uint)B[0]|((uint)B[1]<<8)|((uint)B[2]<<16)|((uint)B[3]<<24));
     }
     else
-      return UINT32(*Addr);
+      return GET_UINT32(*Addr);
 #else
-    return UINT32(*Addr);
+    return GET_UINT32(*Addr);
 #endif
   }
 }
@@ -60,7 +61,7 @@ inline uint RarVM::GetValue(bool ByteMode,uint *Addr)
 #if defined(BIG_ENDIAN) || !defined(ALLOW_NOT_ALIGNED_INT)
   #define GET_VALUE(ByteMode,Addr) GetValue(ByteMode,(uint *)Addr)
 #else
-  #define GET_VALUE(ByteMode,Addr) ((ByteMode) ? (*(byte *)(Addr)):UINT32(*(uint *)(Addr)))
+  #define GET_VALUE(ByteMode,Addr) ((ByteMode) ? (*(byte *)(Addr)):GET_UINT32(*(uint *)(Addr)))
 #endif
 
 
@@ -204,7 +205,7 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,uint CodeSize)
       case VM_CMP:
         {
           uint Value1=GET_VALUE(Cmd->ByteMode,Op1);
-          uint Result=UINT32(Value1-GET_VALUE(Cmd->ByteMode,Op2));
+          uint Result=GET_UINT32(Value1-GET_VALUE(Cmd->ByteMode,Op2));
           Flags=Result==0 ? VM_FZ:(Result>Value1)|(Result&VM_FS);
         }
         break;
@@ -212,14 +213,14 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,uint CodeSize)
       case VM_CMPB:
         {
           uint Value1=GET_VALUE(true,Op1);
-          uint Result=UINT32(Value1-GET_VALUE(true,Op2));
+          uint Result=GET_UINT32(Value1-GET_VALUE(true,Op2));
           Flags=Result==0 ? VM_FZ:(Result>Value1)|(Result&VM_FS);
         }
         break;
       case VM_CMPD:
         {
           uint Value1=GET_VALUE(false,Op1);
-          uint Result=UINT32(Value1-GET_VALUE(false,Op2));
+          uint Result=GET_UINT32(Value1-GET_VALUE(false,Op2));
           Flags=Result==0 ? VM_FZ:(Result>Value1)|(Result&VM_FS);
         }
         break;
@@ -227,7 +228,7 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,uint CodeSize)
       case VM_ADD:
         {
           uint Value1=GET_VALUE(Cmd->ByteMode,Op1);
-          uint Result=UINT32(Value1+GET_VALUE(Cmd->ByteMode,Op2));
+          uint Result=GET_UINT32(Value1+GET_VALUE(Cmd->ByteMode,Op2));
           if (Cmd->ByteMode)
           {
             Result&=0xff;
@@ -249,7 +250,7 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,uint CodeSize)
       case VM_SUB:
         {
           uint Value1=GET_VALUE(Cmd->ByteMode,Op1);
-          uint Result=UINT32(Value1-GET_VALUE(Cmd->ByteMode,Op2));
+          uint Result=GET_UINT32(Value1-GET_VALUE(Cmd->ByteMode,Op2));
           Flags=Result==0 ? VM_FZ:(Result>Value1)|(Result&VM_FS);
           SET_VALUE(Cmd->ByteMode,Op1,Result);
         }
@@ -278,7 +279,7 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,uint CodeSize)
         break;
       case VM_INC:
         {
-          uint Result=UINT32(GET_VALUE(Cmd->ByteMode,Op1)+1);
+          uint Result=GET_UINT32(GET_VALUE(Cmd->ByteMode,Op1)+1);
           if (Cmd->ByteMode)
             Result&=0xff;
           SET_VALUE(Cmd->ByteMode,Op1,Result);
@@ -295,7 +296,7 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,uint CodeSize)
 #endif
       case VM_DEC:
         {
-          uint Result=UINT32(GET_VALUE(Cmd->ByteMode,Op1)-1);
+          uint Result=GET_UINT32(GET_VALUE(Cmd->ByteMode,Op1)-1);
           SET_VALUE(Cmd->ByteMode,Op1,Result);
           Flags=Result==0 ? VM_FZ:Result&VM_FS;
         }
@@ -313,28 +314,28 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,uint CodeSize)
         continue;
       case VM_XOR:
         {
-          uint Result=UINT32(GET_VALUE(Cmd->ByteMode,Op1)^GET_VALUE(Cmd->ByteMode,Op2));
+          uint Result=GET_UINT32(GET_VALUE(Cmd->ByteMode,Op1)^GET_VALUE(Cmd->ByteMode,Op2));
           Flags=Result==0 ? VM_FZ:Result&VM_FS;
           SET_VALUE(Cmd->ByteMode,Op1,Result);
         }
         break;
       case VM_AND:
         {
-          uint Result=UINT32(GET_VALUE(Cmd->ByteMode,Op1)&GET_VALUE(Cmd->ByteMode,Op2));
+          uint Result=GET_UINT32(GET_VALUE(Cmd->ByteMode,Op1)&GET_VALUE(Cmd->ByteMode,Op2));
           Flags=Result==0 ? VM_FZ:Result&VM_FS;
           SET_VALUE(Cmd->ByteMode,Op1,Result);
         }
         break;
       case VM_OR:
         {
-          uint Result=UINT32(GET_VALUE(Cmd->ByteMode,Op1)|GET_VALUE(Cmd->ByteMode,Op2));
+          uint Result=GET_UINT32(GET_VALUE(Cmd->ByteMode,Op1)|GET_VALUE(Cmd->ByteMode,Op2));
           Flags=Result==0 ? VM_FZ:Result&VM_FS;
           SET_VALUE(Cmd->ByteMode,Op1,Result);
         }
         break;
       case VM_TEST:
         {
-          uint Result=UINT32(GET_VALUE(Cmd->ByteMode,Op1)&GET_VALUE(Cmd->ByteMode,Op2));
+          uint Result=GET_UINT32(GET_VALUE(Cmd->ByteMode,Op1)&GET_VALUE(Cmd->ByteMode,Op2));
           Flags=Result==0 ? VM_FZ:Result&VM_FS;
         }
         break;
@@ -400,7 +401,7 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,uint CodeSize)
         {
           uint Value1=GET_VALUE(Cmd->ByteMode,Op1);
           uint Value2=GET_VALUE(Cmd->ByteMode,Op2);
-          uint Result=UINT32(Value1<<Value2);
+          uint Result=GET_UINT32(Value1<<Value2);
           Flags=(Result==0 ? VM_FZ:(Result&VM_FS))|((Value1<<(Value2-1))&0x80000000 ? VM_FC:0);
           SET_VALUE(Cmd->ByteMode,Op1,Result);
         }
@@ -409,7 +410,7 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,uint CodeSize)
         {
           uint Value1=GET_VALUE(Cmd->ByteMode,Op1);
           uint Value2=GET_VALUE(Cmd->ByteMode,Op2);
-          uint Result=UINT32(Value1>>Value2);
+          uint Result=GET_UINT32(Value1>>Value2);
           Flags=(Result==0 ? VM_FZ:(Result&VM_FS))|((Value1>>(Value2-1))&VM_FC);
           SET_VALUE(Cmd->ByteMode,Op1,Result);
         }
@@ -418,7 +419,7 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,uint CodeSize)
         {
           uint Value1=GET_VALUE(Cmd->ByteMode,Op1);
           uint Value2=GET_VALUE(Cmd->ByteMode,Op2);
-          uint Result=UINT32(((int)Value1)>>Value2);
+          uint Result=GET_UINT32(((int)Value1)>>Value2);
           Flags=(Result==0 ? VM_FZ:(Result&VM_FS))|((Value1>>(Value2-1))&VM_FC);
           SET_VALUE(Cmd->ByteMode,Op1,Result);
         }
@@ -427,7 +428,7 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,uint CodeSize)
         {
           // We use "0-value" expression to suppress "unary minus to unsigned"
           // compiler warning.
-          uint Result=UINT32(0-GET_VALUE(Cmd->ByteMode,Op1));
+          uint Result=GET_UINT32(0-GET_VALUE(Cmd->ByteMode,Op1));
           Flags=Result==0 ? VM_FZ:VM_FC|(Result&VM_FS);
           SET_VALUE(Cmd->ByteMode,Op1,Result);
         }
@@ -496,7 +497,7 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,uint CodeSize)
         {
           uint Value1=GET_VALUE(Cmd->ByteMode,Op1);
           uint FC=(Flags&VM_FC);
-          uint Result=UINT32(Value1+GET_VALUE(Cmd->ByteMode,Op2)+FC);
+          uint Result=GET_UINT32(Value1+GET_VALUE(Cmd->ByteMode,Op2)+FC);
           if (Cmd->ByteMode)
             Result&=0xff;
           Flags=(Result<Value1 || Result==Value1 && FC)|(Result==0 ? VM_FZ:(Result&VM_FS));
@@ -507,7 +508,7 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,uint CodeSize)
         {
           uint Value1=GET_VALUE(Cmd->ByteMode,Op1);
           uint FC=(Flags&VM_FC);
-          uint Result=UINT32(Value1-GET_VALUE(Cmd->ByteMode,Op2)-FC);
+          uint Result=GET_UINT32(Value1-GET_VALUE(Cmd->ByteMode,Op2)-FC);
           if (Cmd->ByteMode)
             Result&=0xff;
           Flags=(Result>Value1 || Result==Value1 && FC)|(Result==0 ? VM_FZ:(Result&VM_FS));
@@ -524,7 +525,7 @@ bool RarVM::ExecuteCode(VM_PreparedCommand *PreparedCode,uint CodeSize)
 #ifdef VM_STANDARDFILTERS
       case VM_STANDARD:
         ExecuteStandardFilter((VM_StandardFilters)Cmd->Op1.Data);
-        break;
+        return true;
 #endif
       case VM_PRINT:
         break;
@@ -565,8 +566,10 @@ void RarVM::Prepare(byte *Code,uint CodeSize,VM_PreparedProgram *Prg)
       CurCmd->Op2.Addr=&CurCmd->Op2.Data;
       CurCmd->Op1.Type=CurCmd->Op2.Type=VM_OPNONE;
       CodeSize=0;
+      return;
     }
 #endif  
+#ifndef NORARVM
     uint DataFlag=fgetbits();
     faddbits(1);
 
@@ -640,6 +643,7 @@ void RarVM::Prepare(byte *Code,uint CodeSize,VM_PreparedProgram *Prg)
       }
       Prg->CmdCount++;
     }
+#endif
   }
 
   // Adding RET command at the end of program.
@@ -671,6 +675,7 @@ void RarVM::Prepare(byte *Code,uint CodeSize,VM_PreparedProgram *Prg)
 }
 
 
+#ifndef NORARVM
 void RarVM::DecodeArg(VM_PreparedOperand &Op,bool ByteMode)
 {
   uint Data=fgetbits();
@@ -727,6 +732,7 @@ void RarVM::DecodeArg(VM_PreparedOperand &Op,bool ByteMode)
       }
     }
 }
+#endif
 
 
 uint RarVM::ReadData(BitInput &Inp)
@@ -765,7 +771,7 @@ uint RarVM::ReadData(BitInput &Inp)
 }
 
 
-void RarVM::SetMemory(uint Pos,byte *Data,uint DataSize)
+void RarVM::SetMemory(size_t Pos,byte *Data,size_t DataSize)
 {
   if (Pos<VM_MEMSIZE && Data!=Mem+Pos)
     memmove(Mem+Pos,Data,Min(DataSize,VM_MEMSIZE-Pos));
@@ -855,10 +861,9 @@ VM_StandardFilters RarVM::IsStandardFilter(byte *Code,uint CodeSize)
    120, 0x3769893f, VMSF_ITANIUM,
     29, 0x0e06077d, VMSF_DELTA,
    149, 0x1c2c5dc8, VMSF_RGB,
-   216, 0xbc85e701, VMSF_AUDIO,
-    40, 0x46b9c560, VMSF_UPCASE
+   216, 0xbc85e701, VMSF_AUDIO
   };
-  uint CodeCRC=CRC(0xffffffff,Code,CodeSize)^0xffffffff;
+  uint CodeCRC=CRC32(0xffffffff,Code,CodeSize)^0xffffffff;
   for (uint I=0;I<ASIZE(StdList);I++)
     if (StdList[I].CRC==CodeCRC && StdList[I].Length==CodeSize)
       return(StdList[I].Type);
@@ -877,7 +882,7 @@ void RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
         int DataSize=R[4];
         uint FileOffset=R[6];
 
-        if (DataSize>=VM_GLOBALMEMADDR || DataSize<4)
+        if ((uint)DataSize>=VM_GLOBALMEMADDR || DataSize<4)
           break;
 
         const int FileSize=0x1000000;
@@ -923,7 +928,7 @@ void RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
         int DataSize=R[4];
         uint FileOffset=R[6];
 
-        if (DataSize>=VM_GLOBALMEMADDR || DataSize<21)
+        if ((uint)DataSize>=VM_GLOBALMEMADDR || DataSize<21)
           break;
 
         int CurPos=0;
@@ -960,7 +965,7 @@ void RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
       {
         int DataSize=R[4],Channels=R[0],SrcPos=0,Border=DataSize*2;
         SET_VALUE(false,&Mem[VM_GLOBALMEMADDR+0x20],DataSize);
-        if (DataSize>=VM_GLOBALMEMADDR/2)
+        if ((uint)DataSize>=VM_GLOBALMEMADDR/2)
           break;
 
         // Bytes from same channels are grouped to continual data blocks,
@@ -979,7 +984,7 @@ void RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
         byte *SrcData=Mem,*DestData=SrcData+DataSize;
         const int Channels=3;
         SET_VALUE(false,&Mem[VM_GLOBALMEMADDR+0x20],DataSize);
-        if (DataSize>=VM_GLOBALMEMADDR/2 || PosR<0)
+        if ((uint)DataSize>=VM_GLOBALMEMADDR/2 || PosR<0)
           break;
         for (int CurChannel=0;CurChannel<Channels;CurChannel++)
         {
@@ -1024,7 +1029,7 @@ void RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
         int DataSize=R[4],Channels=R[0];
         byte *SrcData=Mem,*DestData=SrcData+DataSize;
         SET_VALUE(false,&Mem[VM_GLOBALMEMADDR+0x20],DataSize);
-        if (DataSize>=VM_GLOBALMEMADDR/2)
+        if ((uint)DataSize>=VM_GLOBALMEMADDR/2)
           break;
         for (int CurChannel=0;CurChannel<Channels;CurChannel++)
         {
@@ -1086,22 +1091,6 @@ void RarVM::ExecuteStandardFilter(VM_StandardFilters FilterType)
         }
       }
       break;
-    case VMSF_UPCASE:
-      {
-        int DataSize=R[4],SrcPos=0,DestPos=DataSize;
-        if (DataSize>=VM_GLOBALMEMADDR/2)
-          break;
-        while (SrcPos<DataSize)
-        {
-          byte CurByte=Mem[SrcPos++];
-          if (CurByte==2 && (CurByte=Mem[SrcPos++])!=2)
-            CurByte-=32;
-          Mem[DestPos++]=CurByte;
-        }
-        SET_VALUE(false,&Mem[VM_GLOBALMEMADDR+0x1c],DestPos-DataSize);
-        SET_VALUE(false,&Mem[VM_GLOBALMEMADDR+0x20],DataSize);
-      }
-      break;
   }
 }
 
