@@ -37,7 +37,7 @@ class ComprDataIO
     FileHeader *SubHead;
     int64 *SubHeadPos;
 
-#ifndef NOCRYPT
+#ifndef RAR_NOCRYPT
     CryptData Crypt;
     CryptData Decrypt;
 #endif
@@ -45,7 +45,7 @@ class ComprDataIO
 
     int LastPercent;
 
-    char CurrentCommand;
+    wchar CurrentCommand;
 
   public:
     ComprDataIO();
@@ -60,29 +60,32 @@ class ComprDataIO
     void SetFiles(File *SrcFile,File *DestFile);
     void SetCommand(CmdAdd *Cmd) {Command=Cmd;}
     void SetSubHeader(FileHeader *hd,int64 *Pos) {SubHead=hd;SubHeadPos=Pos;}
-    void SetEncryption(int Method,const wchar *Password,const byte *Salt,bool Encrypt,bool HandsOffHash);
+    void SetEncryption(bool Encrypt,CRYPT_METHOD Method,SecPassword *Password,
+         const byte *Salt,const byte *InitV,uint Lg2Cnt,byte *PswCheck,byte *HashKey);
     void SetAV15Encryption();
     void SetCmt13Encryption();
     void SetUnpackToMemory(byte *Addr,uint Size);
-    void SetCurrentCommand(char Cmd) {CurrentCommand=Cmd;}
+    void SetCurrentCommand(wchar Cmd) {CurrentCommand=Cmd;}
 
     bool PackVolume;
     bool UnpVolume;
     bool NextVolumeMissing;
-    int64 TotalPackRead;
     int64 UnpArcSize;
     int64 CurPackRead,CurPackWrite,CurUnpRead,CurUnpWrite;
 
+
     // Size of already processed archives.
     // Used to calculate the total operation progress.
     int64 ProcessedArcSize;
 
     int64 TotalArcSize;
 
-    uint PackFileCRC,UnpFileCRC,PackedCRC;
+    DataHash PackedDataHash; // Packed write and unpack read hash.
+    DataHash PackHash; // Pack read hash.
+    DataHash UnpHash;  // Unpack write hash.
 
-    int Encryption;
-    int Decryption;
+    bool Encryption;
+    bool Decryption;
 };
 
 #endif
