@@ -3,15 +3,10 @@
 
 const char* NullToEmpty(const char *Str);
 const wchar* NullToEmpty(const wchar *Str);
-char* IntNameToExt(const char *Name);
-void ExtToInt(const char *Src,char *Dest);
-void IntToExt(const char *Src,char *Dest);
-char* strlower(char *Str);
-char* strupper(char *Str);
-int stricomp(const char *Str1,const char *Str2);
-int strnicomp(const char *Str1,const char *Str2,size_t N);
-char* RemoveEOL(char *Str);
-char* RemoveLF(char *Str);
+void IntToExt(const char *Src,char *Dest,size_t DestSize);
+int stricomp(const char *s1,const char *s2);
+int strnicomp(const char *s1,const char *s2,size_t n);
+wchar* RemoveEOL(wchar *Str);
 wchar* RemoveLF(wchar *Str);
 unsigned char loctolower(unsigned char ch);
 unsigned char loctoupper(unsigned char ch);
@@ -28,6 +23,7 @@ bool IsDigit(int ch);
 bool IsSpace(int ch);
 bool IsAlpha(int ch);
 
+void BinToHex(const byte *Bin,size_t BinSize,char *Hex,wchar *HexW,size_t HexSize);
 
 #ifndef SFX_MODULE
 uint GetDigits(uint Number);
@@ -36,18 +32,17 @@ uint GetDigits(uint Number);
 bool LowAscii(const char *Str);
 bool LowAscii(const wchar *Str);
 
-
-int stricompc(const char *Str1,const char *Str2);
-#ifndef SFX_MODULE
 int wcsicompc(const wchar *Str1,const wchar *Str2);
-#endif
 
 void itoa(int64 n,char *Str);
-int64 atoil(const char *Str);
 void itoa(int64 n,wchar *Str);
-int64 atoil(const wchar *Str);
 const wchar* GetWide(const char *Src);
-const wchar* GetWide(const char *Src,const wchar *SrcW);
 const wchar* GetCmdParam(const wchar *CmdLine,wchar *Param,size_t MaxSize);
+#ifndef SILENT
+void PrintfPrepareFmt(const wchar *Org,wchar *Cvt,size_t MaxSize);
+#endif
+
+enum PASSWORD_TYPE {PASSWORD_GLOBAL,PASSWORD_FILE,PASSWORD_ARCHIVE};
+bool GetPassword(PASSWORD_TYPE Type,const wchar *FileName,SecPassword *Password);
 
 #endif
