@@ -70,7 +70,7 @@ void SubAllocator::StopSubAllocator()
   if ( SubAllocatorSize ) 
   {
     SubAllocatorSize=0;
-    rarfree(HeapStart);
+    free(HeapStart);
   }
 }
 
@@ -87,7 +87,7 @@ bool SubAllocator::StartSubAllocator(int SASize)
   // units: one as reserve for HeapEnd overflow checks and another
   // to provide the space to correctly align UnitsStart.
   uint AllocSize=t/FIXED_UNIT_SIZE*UNIT_SIZE+2*UNIT_SIZE;
-  if ((HeapStart=(byte *)rarmalloc(AllocSize)) == NULL)
+  if ((HeapStart=(byte *)malloc(AllocSize)) == NULL)
   {
     ErrHandler.MemoryError();
     return FALSE;
