@@ -1,7 +1,20 @@
 #include "rar.hpp"
+#define MBFUNCTIONS
+
+#if defined(_UNIX) && defined(MBFUNCTIONS)
+
+static bool WideToCharMap(const wchar *Src,char *Dest,size_t DestSize,bool &Success);
+static void CharToWideMap(const char *Src,wchar *Dest,size_t DestSize,bool &Success);
+
+// In Unix we map high ASCII characters which cannot be converted to Unicode
+// to 0xE000 - 0xE0FF private use Unicode area.
+static const uint MapAreaStart=0xE000;
+
+// Mapped string marker. Initially we used 0xFFFF for this purpose,
+// but it causes MSVC2008 swprintf to fail (it treats 0xFFFF as error marker).
+// While we could workaround it, it is safer to use another character.
+static const uint MappedStringMark=0xFFFE;
 
-#if defined(_EMX) && !defined(_DJGPP)
-#include "unios2.cpp"
 #endif
 
 bool WideToChar(const wchar *Src,char *Dest,size_t DestSize)
@@ -17,42 +30,25 @@ bool WideToChar(const wchar *Src,char *Dest,size_t DestSize)
   WideToUtf(Src,Dest,DestSize);
 
 #elif defined(MBFUNCTIONS)
-  size_t ResultingSize=wcstombs(Dest,Src,DestSize);
-  if (ResultingSize==(size_t)-1)
-    RetCode=false;
-  if (ResultingSize==0 && *Src!=0)
-    RetCode=false;
-
-  if ((!RetCode || *Dest==0 && *Src!=0) && DestSize>NM && wcslen(Src)<NM)
+  if (!WideToCharMap(Src,Dest,DestSize,RetCode))
   {
-    /* Workaround for strange Linux Unicode functions bug.
-       Some of wcstombs and mbstowcs implementations in some situations
-       (we are yet to find out what it depends on) can return an empty
-       string and success code if buffer size value is too large.
-    */
-    return(WideToChar(Src,Dest,NM));
-  }
-
-#else
-  if (UnicodeEnabled())
-  {
-#if defined(_EMX) && !defined(_DJGPP)
-    int len=Min(wcslen(Src)+1,DestSize-1);
-    if (uni_fromucs((UniChar*)Src,len,Dest,(size_t*)&DestSize)==-1 ||
-        DestSize>len*2)
+    size_t ResultingSize=wcstombs(Dest,Src,DestSize);
+    if (ResultingSize==(size_t)-1)
+      RetCode=false;
+    if (ResultingSize==0 && *Src!=0)
       RetCode=false;
-    Dest[DestSize]=0;
-#endif
   }
-  else
-    for (int I=0;I<DestSize;I++)
-    {
-      Dest[I]=(char)Src[I];
-      if (Src[I]==0)
-        break;
-    }
+#else
+  for (int I=0;I<DestSize;I++)
+  {
+    Dest[I]=(char)Src[I];
+    if (Src[I]==0)
+      break;
+  }
 #endif
-
+  if (DestSize>0)
+    Dest[DestSize-1]=0;
+  
   // We tried to return the empty string if conversion is failed,
   // but it does not work well. WideCharToMultiByte returns 'failed' code
   // and partially converted string even if we wanted to convert only a part
@@ -60,7 +56,7 @@ bool WideToChar(const wchar *Src,char *Dest,size_t DestSize)
   // string. Such call is the valid behavior in RAR code and we do not expect
   // the empty string in this case.
 
-  return(RetCode);
+  return RetCode;
 }
 
 
@@ -83,34 +79,19 @@ bool CharToWide(const char *Src,wchar *Dest,size_t DestSize)
   if (ResultingSize==0 && *Src!=0)
     RetCode=false;
 
-  if ((!RetCode || *Dest==0 && *Src!=0) && DestSize>NM && strlen(Src)<NM)
-  {
-    /* Workaround for strange Linux Unicode functions bug.
-       Some of wcstombs and mbstowcs implementations in some situations
-       (we are yet to find out what it depends on) can return an empty
-       string and success code if buffer size value is too large.
-    */
-    return(CharToWide(Src,Dest,NM));
-  }
+  if (RetCode==false && DestSize>1)
+    CharToWideMap(Src,Dest,DestSize,RetCode);
+
 #else
-  if (UnicodeEnabled())
+  for (int I=0;I<DestSize;I++)
   {
-#if defined(_EMX) && !defined(_DJGPP)
-    int len=Min(strlen(Src)+1,DestSize-1);
-    if (uni_toucs((char*)Src,len,(UniChar*)Dest,(size_t*)&DestSize)==-1 ||
-        DestSize>len)
-      DestSize=0;
-    RetCode=false;
-#endif
+    Dest[I]=(wchar_t)Src[I];
+    if (Src[I]==0)
+      break;
   }
-  else
-    for (int I=0;I<DestSize;I++)
-    {
-      Dest[I]=(wchar_t)Src[I];
-      if (Src[I]==0)
-        break;
-    }
 #endif
+  if (DestSize>0)
+    Dest[DestSize-1]=0;
 
   // We tried to return the empty string if conversion is failed,
   // but it does not work well. MultiByteToWideChar returns 'failed' code
@@ -118,10 +99,104 @@ bool CharToWide(const char *Src,wchar *Dest,size_t DestSize)
   // smaller than required for fully converted string. Such call is the valid
   // behavior in RAR code and we do not expect the empty string in this case.
 
-  return(RetCode);
+  return RetCode;
 }
 
 
+#if defined(_UNIX) && defined(MBFUNCTIONS)
+// Convert and restore mapped inconvertible Unicode characters. 
+// We use it for extended ASCII names in Unix.
+bool WideToCharMap(const wchar *Src,char *Dest,size_t DestSize,bool &Success)
+{
+  // String with inconvertible characters mapped to private use Unicode area
+  // must have the mark code somewhere.
+  if (wcschr(Src,(wchar)MappedStringMark)==NULL)
+    return false;
+
+  Success=true;
+  uint SrcPos=0,DestPos=0;
+  while (DestPos<DestSize-MB_CUR_MAX)
+  {
+    if (Src[SrcPos]==0)
+    {
+      Dest[DestPos]=0;
+      break;
+    }
+    if (uint(Src[SrcPos])==MappedStringMark)
+    {
+      SrcPos++;
+      continue;
+    }
+    // For security reasons do not retore low ASCII codes, so mapping cannot
+    // be used to hide control codes like path separators.
+    if (uint(Src[SrcPos])>=MapAreaStart+0x80 && uint(Src[SrcPos])<MapAreaStart+0x100)
+      Dest[DestPos++]=char(uint(Src[SrcPos++])-MapAreaStart);
+    else
+    {
+      ignore_result( wctomb(NULL,0) ); // Reset shift state.
+      if (wctomb(Dest+DestPos,Src[SrcPos])==-1)
+        Success=false;
+      SrcPos++;
+      ignore_result( mblen(NULL,0) ); // Reset shift state.
+      int Length=mblen(Dest+DestPos,MB_CUR_MAX);
+      DestPos+=Max(Length,1);
+    }
+  }
+  return true;
+}
+#endif
+
+
+#if defined(_UNIX) && defined(MBFUNCTIONS)
+// Convert and map inconvertible Unicode characters. 
+// We use it for extended ASCII names in Unix.
+void CharToWideMap(const char *Src,wchar *Dest,size_t DestSize,bool &Success)
+{
+  // Map inconvertible characters to private use Unicode area 0xE000.
+  // Mark such string by placing special non-character code before
+  // first inconvertible character.
+  Success=false;
+  bool MarkAdded=false;
+  uint SrcPos=0,DestPos=0;
+  while (DestPos<DestSize)
+  {
+    if (Src[SrcPos]==0)
+    {
+      Dest[DestPos]=0;
+      Success=true;
+      break;
+    }
+    ignore_result( mbtowc(NULL,NULL,0) ); // Reset shift state.
+    if (mbtowc(Dest+DestPos,Src+SrcPos,MB_CUR_MAX)==-1)
+    {
+      // For security reasons we do not want to map low ASCII characters,
+      // so we do not have additional .. and path separator codes.
+      if (byte(Src[SrcPos])>=0x80)
+      {
+        if (!MarkAdded)
+        {
+          Dest[DestPos++]=MappedStringMark;
+          MarkAdded=true;
+          if (DestPos>=DestSize)
+            break;
+        }
+        Dest[DestPos++]=byte(Src[SrcPos++])+MapAreaStart;
+      }
+      else
+        break;
+    }
+    else
+    {
+      ignore_result( mblen(NULL,0) ); // Reset shift state.
+      int Length=mblen(Src+SrcPos,MB_CUR_MAX);
+      SrcPos+=Max(Length,1);
+      DestPos++;
+    }
+  }
+}
+#endif
+
+
 // SrcSize is in wide characters, not in bytes.
 byte* WideToRaw(const wchar *Src,byte *Dest,size_t SrcSize)
 {
@@ -132,7 +207,7 @@ byte* WideToRaw(const wchar *Src,byte *Dest,size_t SrcSize)
     if (*Src==0)
       break;
   }
-  return(Dest);
+  return Dest;
 }
 
 
@@ -141,7 +216,7 @@ wchar* RawToWide(const byte *Src,wchar *Dest,size_t DestSize)
   for (size_t I=0;I<DestSize;I++)
     if ((Dest[I]=Src[I*2]+(Src[I*2+1]<<8))==0)
       break;
-  return(Dest);
+  return Dest;
 }
 
 
@@ -161,6 +236,12 @@ void WideToUtf(const wchar *Src,char *Dest,size_t DestSize)
         *(Dest++)=(0x80|(c&0x3f));
       }
       else
+      {
+        if (c>=0xd800 && c<=0xdbff && *Src>=0xdc00 && *Src<=0xdfff) // Surrogate pair.
+        {
+          c=((c-0xd800)<<10)+(*Src-0xdc00)+0x10000;
+          Src++;
+        }
         if (c<0x10000 && (dsize-=2)>=0)
         {
           *(Dest++)=(0xe0|(c>>12));
@@ -175,11 +256,39 @@ void WideToUtf(const wchar *Src,char *Dest,size_t DestSize)
             *(Dest++)=(0x80|((c>>6)&0x3f));
             *(Dest++)=(0x80|(c&0x3f));
           }
+      }
   }
   *Dest=0;
 }
 
 
+size_t WideToUtfSize(const wchar *Src)
+{
+  size_t Size=0;
+  for (;*Src!=0;Src++)
+    if (*Src<0x80)
+      Size++;
+    else
+      if (*Src<0x800)
+        Size+=2;
+      else
+        if (*Src<0x10000)
+        {
+          if (Src[0]>=0xd800 && Src[0]<=0xdbff && Src[1]>=0xdc00 && Src[1]<=0xdfff)
+          {
+            Size+=4; // 4 output bytes for Unicode surrogate pair.
+            Src++;
+          }
+          else
+            Size+=3;
+        }
+        else
+          if (*Src<0x200000)
+            Size+=4;
+  return Size+1; // Include terminating zero.
+}
+
+
 // Dest can be NULL if we only need to check validity of Src.
 bool UtfToWide(const char *Src,wchar *Dest,size_t DestSize)
 {
@@ -188,14 +297,17 @@ bool UtfToWide(const char *Src,wchar *Dest,size_t DestSize)
   dsize--;
   while (*Src!=0)
   {
-    uint c=(byte)*(Src++),d;
+    uint c=byte(*(Src++)),d;
     if (c<0x80)
       d=c;
     else
       if ((c>>5)==6)
       {
         if ((*Src&0xc0)!=0x80)
+        {
+          Success=false;
           break;
+        }
         d=((c&0x1f)<<6)|(*Src&0x3f);
         Src++;
       }
@@ -203,7 +315,10 @@ bool UtfToWide(const char *Src,wchar *Dest,size_t DestSize)
         if ((c>>4)==14)
         {
           if ((Src[0]&0xc0)!=0x80 || (Src[1]&0xc0)!=0x80)
+          {
+            Success=false;
             break;
+          }
           d=((c&0xf)<<12)|((Src[0]&0x3f)<<6)|(Src[1]&0x3f);
           Src+=2;
         }
@@ -211,16 +326,17 @@ bool UtfToWide(const char *Src,wchar *Dest,size_t DestSize)
           if ((c>>3)==30)
           {
             if ((Src[0]&0xc0)!=0x80 || (Src[1]&0xc0)!=0x80 || (Src[2]&0xc0)!=0x80)
+            {
+              Success=false;
               break;
+            }
             d=((c&7)<<18)|((Src[0]&0x3f)<<12)|((Src[1]&0x3f)<<6)|(Src[2]&0x3f);
             Src+=3;
           }
           else
           {
-            // Skip bad character, but continue processing, so we can handle
-            // archived UTF-8 file names even if one of characters is corrupt.
             Success=false;
-            continue;
+            break;
           }
     if (Dest!=NULL && --dsize<0)
       break;
@@ -228,17 +344,19 @@ bool UtfToWide(const char *Src,wchar *Dest,size_t DestSize)
     {
       if (Dest!=NULL && --dsize<0)
         break;
-      if (d>0x10ffff)
+      if (d>0x10ffff) // UTF-8 must end at 0x10ffff according to RFC 3629.
       {
-        // UTF-8 is restricted by RFC 3629 to end at 0x10ffff.
         Success=false;
         continue;
       }
       if (Dest!=NULL)
-      {
-        *(Dest++)=((d-0x10000)>>10)+0xd800;
-        *(Dest++)=(d&0x3ff)+0xdc00;
-      }
+        if (sizeof(*Dest)==2) // Use the surrogate pair for 2 byte Unicode.
+        {
+          *(Dest++)=((d-0x10000)>>10)+0xd800;
+          *(Dest++)=(d&0x3ff)+0xdc00;
+        }
+        else
+          *(Dest++)=d;
     }
     else
       if (Dest!=NULL)
@@ -250,91 +368,110 @@ bool UtfToWide(const char *Src,wchar *Dest,size_t DestSize)
 }
 
 
-bool UnicodeEnabled()
-{
-#ifdef UNICODE_SUPPORTED
-  #ifdef _EMX
-    return(uni_ready);
-  #else
-    return(true);
-  #endif
-#else
-  return(false);
-#endif
-}
-
-
 int wcsicomp(const wchar *s1,const wchar *s2)
 {
-  char Ansi1[NM*sizeof(wchar)],Ansi2[NM*sizeof(wchar)];
-  WideToChar(s1,Ansi1,sizeof(Ansi1));
-  WideToChar(s2,Ansi2,sizeof(Ansi2));
-  return(stricomp(Ansi1,Ansi2));
-}
-
-
-static int wcsnicomp_w2c(const wchar *s1,const wchar *s2,size_t n)
-{
-  char Ansi1[NM*2],Ansi2[NM*2];
-  GetAsciiName(s1,Ansi1,ASIZE(Ansi1));
-  GetAsciiName(s2,Ansi2,ASIZE(Ansi2));
-  return(stricomp(Ansi1,Ansi2));
+#ifdef _WIN_ALL
+  return CompareStringW(LOCALE_USER_DEFAULT,NORM_IGNORECASE|SORT_STRINGSORT,s1,-1,s2,-1)-2;
+#else
+  while (towupper(*s1)==towupper(*s2))
+  {
+    if (*s1==0)
+      return 0;
+    s1++;
+    s2++;
+  }
+  return s1 < s2 ? -1 : 1;
+#endif
 }
 
 
 int wcsnicomp(const wchar *s1,const wchar *s2,size_t n)
 {
-  return(wcsnicomp_w2c(s1,s2,n));
+#ifdef _WIN_ALL
+  // If we specify 'n' exceeding the actual string length, CompareString goes
+  // beyond the trailing zero and compares garbage. So we need to limit 'n'
+  // to real string length.
+  size_t l1=Min(wcslen(s1)+1,n);
+  size_t l2=Min(wcslen(s2)+1,n);
+  return(CompareStringW(LOCALE_USER_DEFAULT,NORM_IGNORECASE|SORT_STRINGSORT,s1,(int)l1,s2,(int)l2)-2);
+#else
+  if (n==0)
+    return 0;
+  while (towupper(*s1)==towupper(*s2))
+  {
+    if (*s1==0 || --n==0)
+      return 0;
+    s1++;
+    s2++;
+  }
+  return s1 < s2 ? -1 : 1;
+#endif
 }
 
 
 #ifndef SFX_MODULE
-wchar* wcslower(wchar *Str)
+wchar* wcslower(wchar *s)
 {
-  for (wchar *ChPtr=Str;*ChPtr;ChPtr++)
-    if (*ChPtr<128)
-      *ChPtr=loctolower((byte)*ChPtr);
-  return(Str);
+#ifdef _WIN_ALL
+  CharLowerW(s);
+#else
+  for (wchar *c=s;*c!=0;c++)
+    *c=towlower(*c);
+#endif
+  return s;
 }
 #endif
 
 
 #ifndef SFX_MODULE
-wchar* wcsupper(wchar *Str)
+wchar* wcsupper(wchar *s)
 {
-  for (wchar *ChPtr=Str;*ChPtr;ChPtr++)
-    if (*ChPtr<128)
-      *ChPtr=loctoupper((byte)*ChPtr);
-  return(Str);
+#ifdef _WIN_ALL
+  CharUpperW(s);
+#else
+  for (wchar *c=s;*c!=0;c++)
+    *c=towupper(*c);
+#endif
+  return s;
 }
 #endif
 
 
 int toupperw(int ch)
 {
-  return((ch<128) ? loctoupper(ch):ch);
+#ifdef _WIN_ALL
+  return (int)CharUpperW((wchar *)ch);
+#else
+  return towupper(ch);
+#endif
 }
 
 
 int tolowerw(int ch)
 {
 #ifdef _WIN_ALL
-  return((int)(LPARAM)CharLowerW((wchar *)(uint)ch));
+  return (int)CharLowerW((wchar *)ch);
 #else
-  return((ch<128) ? loctolower(ch):ch);
+  return towlower(ch);
 #endif
 }
 
 
-int atoiw(const wchar *s)
+uint atoiw(const wchar *s)
 {
-  int n=0;
+  return (uint)atoilw(s);
+}
+
+
+uint64 atoilw(const wchar *s)
+{
+  uint64 n=0;
   while (*s>='0' && *s<='9')
   {
     n=n*10+(*s-'0');
     s++;
   }
-  return(n);
+  return n;
 }
 
 
